//***************************************************************
//	File :		FaxSTFLib.h
//	Authors :	Steve Kirk &Scott Hemme
//	Created :	Thur, Feb 1, 1996
//	CopyRight:	1987 - 1996 STF Technologies, Inc.
//				All Rights Reserved
//  	Notes: 	The libraries were compiled as follows.
//			4 byte int's -- OFF
//			8 byte doubles -- OFF
//***************************************************************

#ifndef __FAXSTFLIB_32__
#define __FAXSTFLIB_32__


//****************************************************************
//  Definitions
//****************************************************************

#define saveHeader 		saveCoverPage
#define setHeader 		setCoverPage
#define restoreHeader 	restoreCoverPage

enum {
	kIdle = 0,
	kSending,
	kReceiving
};

//****************************************************************
//  Errors
//****************************************************************

#define kSTFToolBoxNotFound		-9000			// STF Toolbox not found
#define kSTFToolBoxBusy			-9001			// STF Toolbox could not be opened read/write
#define kSTFSettingsNotFound	-9002			// STF Settings file not found
#define kSTFSettingsFileBusy	-9003			// STF Settings file could not be opened read/write
#define kSTFNoGXSupport			-9004			// FaxPrint GX 3.3 or greater not installed***
#define kSTFBadIndex			-9005			// Index out of range
#define kSTFBrdDrvrFail			-9006			//  Call to the STF Board Driver failed
#define kSTFMonitorNotFound 	-9007			// STF FaxMonitor not found
#define kGetPrinterFail			-9008			// Failed to get the printer name
#define KFaxPrintGXNotFound		-9009			// FaxPrint GX not found
#define kFailtoSwitchDTP		-9010			// Failed to switch the desktop printer (GX only)

//****************************************************************
//  Prototypes
//****************************************************************

// addressing calls
//==========================================
pascal OSErr startup(Str255 note, long sendTime, Boolean autoDeleteFax, Boolean callGroupFax, short cvPgId, Str63 archiveFolder);
pascal OSErr addDestinations (Str31 sendToFaxPhone, Str31 sendToVoicePhone, Str31 sendToCoName, Str31 sendToFirstName,
															 Str31 sendToLastName, Str15 sendToSalutation);
pascal OSErr shutDown(void);

// Manual send/receive
//==========================================
pascal OSErr ManualSend (FSSpecPtr theFile);
pascal OSErr ManualReceive (void);

// switch printer calls
//==========================================
pascal OSErr doSwitchPrinter (Str31 NewPrinter, Str31 OldPrinter);
pascal OSErr doSwitchToFaxPrint (Str31 oldPrinter);
pascal OSErr doRestorePrinter(void);

// other stuff
//==========================================
pascal OSErr doGetSoftwareState(Boolean *flag);
pascal OSErr doScanSpoolFolder(void);
pascal OSErr isFaxSTFInstalled(short *stfInstalled);
pascal OSErr getSTFDriverStatus (short *status);			// -- status = idle(0), sending(1), receiving(2)

// switch settings files calls
//==========================================
pascal OSErr saveCurSettingsFile(void);
pascal OSErr switchSettingsFiles(Str255 newSettingsFileName);
pascal OSErr restoreSettingsFile(void);

// settings calls
//==========================================
// -- CallBlocking settings panel routines
pascal OSErr saveCallBlockingState(void);
pascal OSErr setCallBlockingState(Boolean state, Str63 number);		// if (number == "\p") then this routine only sets the state
pascal OSErr restoreCallBlockingState(void);

// -- Connection settings panel routines
pascal OSErr saveConnectionSettings(void);
pascal OSErr setConnectionSettings(Str63 portName);
pascal OSErr restoreConnectionSettings(void);

// -- CoverPage settings panel routines
pascal OSErr saveCoverPage(void);													// same as saveHeader
pascal OSErr setCoverPage(Str63 name,Str63 company,Str63 faxPhone,Str63 voicePhone);		// same as setHeader
pascal OSErr restoreCoverPage(void);												// same as restoreHeader

// -- Dialing settings panel routines
pascal OSErr saveDialingSettings(void);
pascal OSErr setDialingSettings(short phone,short detect,short carrier,short commaPause,Str63 prefix,Str63 suffix,Str63 macro);
pascal OSErr restoreDialingSettings(void);

// -- FaxForward settings panel routines
pascal OSErr saveFaxForwardState(void);
pascal OSErr setFaxForwardState(Boolean state, Str63 number);	// pass "\p" for number to only toggle the state
pascal OSErr restoreFaxForwardState(void);

// -- FaxIn settings panel routines
pascal OSErr saveFaxInState(void);
pascal OSErr setFaxInState(short processType, short day, short hour, short minute);
pascal OSErr restoreFaxInState(void);

// -- FaxMenu settings panel routines
pascal OSErr saveFaxMenuSettings(void);
pascal OSErr setFaxMenu(Boolean shiftKey, Boolean commandKey, Boolean optionKey);
pascal OSErr restoreFaxMenuSettings(void);

// -- FaxModem settings panel routines
// -- Never(1),Connect(2),Handshake(3),Always(4)
// -- volume		 Low(1),Medium(2),High(3)
// -- answerOn	Number of rings to answer on plus one.
// -- sendSpeed	2400(1),4800(2),7200(3),9600(4),12000(5),14400(6)
// -- recvSpeed	2400(1),4800(2),7200(3),9600(4),12000(5),14400(6)
pascal OSErr saveFaxModemSettings(void);
pascal OSErr setFaxModemSettings(short speakerOn, short volume, short answerOn, short sendSpeed, short recvSpeed);
pascal OSErr restoreFaxModemSettings(void);

// -- FaxQueue settings panel routines
pascal OSErr saveFaxQueueSettings(void);
pascal OSErr setFaxQueueSettings(short sampleRate, short sampleSize, short callGrouping);
pascal OSErr restoreFaxQueueSettings(void);

// -- FaxSoftware settings panel routines
pascal OSErr saveFaxSoftwareSettings(void);
pascal OSErr setFaxSoftwareSettings(short callRetries, short retryInterval, Boolean PageHeader, Boolean confirmTrans,
												Boolean notifyRecept, Boolean faxStatus, Str27 stationMessage);
pascal OSErr pageHeaderOn(void);
pascal OSErr pageHeaderOff(void);
pascal OSErr faxOn(void);
pascal OSErr faxOff(void);
pascal OSErr restoreFaxSoftwareSettings(void);

// -- ModemInit settings panel routines
pascal OSErr saveModemInitSettings(void);
pascal OSErr setModemInitSettings(Str255 initString);
pascal OSErr restoreModemInitSettings(void);

// -- PageHeader settings panel routines
pascal OSErr savePageHeader(void);
pascal OSErr setPageHeaderOpts(Boolean icons,Boolean date,Boolean pages,Boolean time);
pascal OSErr setPageHeaderStandard(Boolean voice, Boolean orginization);
pascal OSErr setPageHeaderCustom(Str63  custHeader);
pascal OSErr restorePageHeader(void);

// -- Places settings panel routines
pascal OSErr savePlacesState(void);
pascal OSErr setPlacesState(short index);						// -- pass -1 in index to turn off, 0 - 29 to select an entry in the location list (1 - 30)
pascal OSErr restorePlacesState(void);

// -- Network settings panel routines
pascal OSErr saveNetworkState(void);
pascal OSErr setNetworkState(Boolean state);
pascal OSErr restoreNetworkState(void);

// -- Security settings panel routines
pascal OSErr saveSecurityState(void);
pascal OSErr setSecurityState(Boolean state, Str63 password);
pascal OSErr restoreSecurityState(void);

// -- TollSaver settings panel routines
pascal OSErr saveTollSaver(void);
pascal OSErr setTollSaverState(Boolean state);
pascal OSErr setTollSaverTimes(short beginHour, short beginMinute, short endHour, short endMinute);
pascal OSErr restoreTollSaver(void);

#endif
